#include "vectorizationPaint.h"
#include <math.h>

void ImagePaint::computeSkeletonGraph(skeleton::SkeletonGraph &sktGraph)
{
	//start point has been pushed back into sktGraph

	if( !QRectF(0.0,0.0,float(m_imgSize.width()-1),float(m_imgSize.height()-1)).contains(m_currPosImg) )
		return;
	/// get information from GPU textures
	_tex[T_GRADIENT]->bind();
	float *pGrad = new float[m_imgSize.width()*m_imgSize.height()*4];
	glGetTexImage(GL_TEXTURE_2D,0,GL_RGBA,GL_FLOAT,pGrad);
	
	_tex[T_GRADIENT_S]->bind();
	float *pGradS = new float[m_imgSize.width()*m_imgSize.height()*4];
	glGetTexImage(GL_TEXTURE_2D,0,GL_RGBA,GL_FLOAT,pGradS);

	_tex[T_PROFILE]->bind();
	float *pProf = new float[m_imgSize.width()*m_imgSize.height()*4];
	glGetTexImage(GL_TEXTURE_2D,0,GL_RGBA,GL_FLOAT,pProf);

	_tex[T_PROJECT]->bind();
	float *pProj = new float[m_imgSize.width()*m_imgSize.height()*4];
	glGetTexImage(GL_TEXTURE_2D,0,GL_RGBA,GL_FLOAT,pProj);

	ImagePaint::tracking(sktGraph, m_imgSize.width(),m_imgSize.height(),4,pGradS,200);
		
	/// delete texture information
	delete []pGrad;
	delete []pGradS;
	delete []pProf;
	delete []pProj;
}

int _clamp(int vmin, int vmax, int v)
{
	return min(max(v,vmin), vmax);
}
float module(float x,float y)
{
	return sqrt(double(x*x+y*y));
}

float Dist(float x1, float y1, float x2, float y2)
{
	return sqrt(pow(x1-x2,2)+pow(y1-y2,2));
}

bool judgeJunctionArea(float *leftPos,float *rightPos)
{
	int i;
	for (i=0;i<=8;i+=2)
	{
		if (leftPos[i]!=0&&rightPos[i]!=0) break;
	}
	float leftx=leftPos[8]-leftPos[i];
	float lefty=leftPos[9]-leftPos[i+1];///ʵӦֿǵjunction point,ӽĻڵend point
	float rightx=rightPos[8]-rightPos[i];
	float righty=rightPos[9]-rightPos[i+1];
	float cosine = (leftx*rightx + lefty*righty)/module(leftx,lefty)/module(rightx,righty);
	if (abs(cosine)<sqrt(double(3))/2)
		if (Dist(leftPos[i],leftPos[i+1],rightPos[i],rightPos[i+1])<Dist(leftPos[8],leftPos[9],rightPos[8],rightPos[9]))//cos30
	{
		return true;//directions we got from left and right boundaries are too different ==> junction area
	}
	return false;
}

const float* getDirection(int w, int h, int cn, const float*flow,  int px, int py)
{
	const float *f = flow + _clamp(0,h-1,/*h-1-*/py)*w*cn + _clamp(0,w-1,px)*cn;
	return f;
}

bool isInImage(float px, float py, int w, int h)
{
	return (px>=0.0) && (px<=float(w-1)) && (py>=0.0) && (py<=float(h-1));
}
void findStepPosition(float *stepPos, float px, float py, int w, int h,
					  int proWidth, const float *pGradS)
{
	const int pxi = int(px+0.5);
	const int pyi = int(py+0.5);
	const float *pGS = pGradS + 4*(pyi*w+pxi);
	
	int distMax = 0;
	float magMax = 0.0;

	/// maximal magnitude on the positive side
	for(int k=0; k<=proWidth; k++)
	{
		int xx = int(float(pxi)+float(k)*pGS[0]);
		int yy = int(float(pyi)+float(k)*pGS[1]);
		if (isInImage(xx,yy,w,h))
		{
			float mag = pGradS[4*(yy*w+xx)+3];
			if(abs(mag)>=magMax)
			{
				distMax = k;
				magMax = abs(mag);
			}
		}
	}
	if (distMax==0) 
	{
		stepPos[0]=-1;
		stepPos[1]=-1;
	}else
	{
		stepPos[0] = float(distMax)*pGS[0]+float(pxi);
		stepPos[1] = float(distMax)*pGS[1]+float(pyi);
	}
	distMax = 0;
	magMax = 0.0;
	/// maximal magnitude on the negative side	
	for(int k=0; k<=proWidth; k++)
	{
		int xx = int(float(pxi)-float(k)*pGS[0]);
		int yy = int(float(pyi)-float(k)*pGS[1]);
		if (isInImage(xx,yy,w,h))
		{
			float mag = pGradS[4*(yy*w+xx)+3];
			if(abs(mag)>=magMax)
			{
				distMax = -k;
				magMax = abs(mag);
			}
		}
	}
	if (distMax==0)
	{
		stepPos[4]=-1;
		stepPos[5]=-1;
	}else
	{
		stepPos[4] = float(distMax)*pGS[0]+float(pxi);
		stepPos[5] = float(distMax)*pGS[1]+float(pyi);
	}
}

void ImagePaint::centerDirection(float* tang, int w, int h, int cn, float px, float py, float *flow)
{
	float stepPos[8];
	const int proWidth = 8;
	findStepPosition(stepPos,px,py,w,h,proWidth, flow);
	float fleftx,flefty,frightx,frighty;
	if (isInImage(stepPos[0],stepPos[1],w,h))
	{
	    const float *fleft = getDirection(w, h, cn,flow,int(stepPos[0]+0.5),int(stepPos[1]+0.5));
	    fleftx=fleft[1];
	    flefty=-fleft[0];
	}else
	{
		const float *fleft = getDirection(w, h, cn,flow,int(px+0.5),int(py+0.5));
		fleftx=fleft[1];
	    flefty=-fleft[0];

	}
	if (!isInImage(px+fleftx,py+flefty,w,h))
	{
		fleftx= -fleftx;
		flefty= -flefty;
	}
	if (isInImage(stepPos[4],stepPos[5],w,h))
	{
		const float *fright = getDirection(w,h, 4,flow,int(stepPos[4]+0.5),int(stepPos[5]+0.5));
		frightx=fright[1];
		frighty=-fright[0];
	}else
	{
		const float *fright = getDirection(w,h, 4,flow,int(px+0.5),int(py+0.5));
		frightx=fright[1];
		frighty=-fright[0];
	}
	if (!isInImage(px+frightx,py+frighty,w,h))
	{
		frightx= -frightx;
		frighty= -frighty;
	}
	if (fleftx*frightx+flefty*frighty<0)
	{
		frightx=-frightx;
		frighty=-frighty;
	}
	//float tang[2];
	tang[0]=(fleftx+frightx)/2.0;
	tang[1]=(flefty+frighty)/2.0;
	//return tang;
}

bool judgeJunctionArea(float leftx,float lefty,float rightx,float righty,float seedx,float seedy)
{
	float dist1=Dist(leftx,lefty,seedx,seedy);
    float dist2=Dist(rightx,righty,seedx,seedy);
	if (dist1>10 && dist1>3*dist2 || dist2>10 && dist2>3*dist1)
		return true;
	return false;
}
bool judgeJunctionArea(float fleftx,float flefty,float frightx,float frighty)
{
	float cosine=(fleftx*frightx+flefty*frighty)/Dist(fleftx,flefty,0,0)/Dist(frightx,frighty,0,0);
	if (cosine>=cos(3.1415/10))
		return false;
	return true;//junctionArea
}
bool findBoundary(float *stepPos, float px, float py, int w, int h,float tangx,float tangy,
	int proWidth, const float *pGradS,QImage m_image)
{
	const int pxi = int(px+0.5);
	const int pyi = int(py+0.5);
	const float *pGS = pGradS + 4*(pyi*w+pxi);

	int distMax = 0;
	float magMax = 0.0;
	bool flag=false;
	/// maximal magnitude on the positive side
	for(int k=0; k<=proWidth; k++)
	{
		int xx = int(float(pxi)-float(k)*tangy);
		int yy = int(float(pyi)+float(k)*tangx);
		if (!isInImage(xx,yy,w,h))
		{//if out of the image, we try to keep the streamline still
			flag=true;
			break;
		}
		float mag = pGradS[4*(yy*w+xx)+3];
		if(abs(mag)>=magMax)
			if (qRed(m_image.pixel(xx,yy))>=qRed(m_image.pixel(int(xx-tangy+0.5),int(yy+tangx+0.5))))
			{
				distMax = k;
				magMax = abs(mag);
			}else
			{
				if (distMax>5)//ʼˣͷ
				{
					break;
				}
			}
		if (distMax!=0 && qRed(m_image.pixel(xx,yy))<10)//
			break;
	}
	if (flag)//ˣʱ߽ܲ׼ȷӦͨж֤ʱұ߽粻ʹ
	{
		return false;
	}else
	{
		stepPos[0] = -float(distMax)*tangy+float(pxi);
		stepPos[1] = float(distMax)*tangx+float(pyi);
		stepPos[2] = float(distMax);
		stepPos[3] = magMax;
		if (qRed(m_image.pixel(stepPos[0]-2*tangy,stepPos[1]+2*tangx))>50)///////˵߽绹Щɣп·һ
			flag=true;
	}
	distMax = 0;
	magMax = 0.0;
	flag=false;
	/// maximal magnitude on the negative side	
	for(int k=0; k<=proWidth; k++)
	{
		int xx = int(float(pxi)+float(k)*tangy);
		int yy = int(float(pyi)-float(k)*tangx);
		if (!isInImage(xx,yy,w,h))
		{
			flag=true;
			break;
		}
		float mag = pGradS[4*(yy*w+xx)+3];
		if(abs(mag)>=magMax)
			if (qRed(m_image.pixel(xx,yy))>=qRed(m_image.pixel(int(0.5+xx+tangy),int(0.5+yy-tangx))))
			{
				distMax = -k;
				magMax = abs(mag);
			}else
			{
				if (distMax>5)//ʼˣͷ
				{
					break;
				}
			}
		if (distMax!=0 && qRed(m_image.pixel(xx,yy))<10)
			break;
	}
	if (flag)
	{
		return false;
	}else
	{
		stepPos[4] = -float(distMax)*tangy+float(pxi);
		stepPos[5] = float(distMax)*tangx+float(pyi);
		stepPos[6] = float(distMax);
		stepPos[7] = magMax;
		if (qRed(m_image.pixel(stepPos[4]+2*tangy,stepPos[5]-2*tangx))>50)
			return false;
	}
	return true;
}

bool judgeEdgeEnd(float px,float py, float tangx, float tangy, int w,int h, int radius,QImage m_image)
{
	if (!isInImage(px+tangx*radius,py+tangy*radius,w,h)||qRed(m_image.pixel(px+tangx*radius,py+tangy*radius))==0)
		return true;
	return false;

}


void ImagePaint::tracking(skeleton::SkeletonGraph &sktGraph, int w, int h, int cn, float *flow, int maxLength)
{
	/////////////////////////////////////////////// tracing along the vector field
	
	///a stack to record the junction points with edges haven't been searched
	vector<skeleton::Vertex> vertexStack,vertexRecord;//second one means angle
    vector<QPointF> angleStack,angleRecord;
	
	/// get the original direction
	//QPointF startPos=QPointF(sktGraph.m_vVertex[0].m_pos.x(), sktGraph.m_vVertex[0].m_pos.y());
	float tang[2];
	m_record=m_image;/////

	centerDirection(tang, m_imgSize.width(),m_imgSize.height(),4,sktGraph.m_vVertex[0].m_pos.x(), sktGraph.m_vVertex[0].m_pos.y(),flow);
	vertexStack.push_back(sktGraph.m_vVertex[0]);
	angleStack.push_back(QPointF(-tang[0],-tang[1]));//make sure it goes both ways
	vertexStack.push_back(sktGraph.m_vVertex[0]);
	angleStack.push_back(QPointF(tang[0],tang[1]));
	int cnt=0;
	while(!vertexStack.empty()&&cnt<=maxLength)//////////////////////
	{
		cnt++;
		cout<<cnt<<"     ";
		///ջ
		vertexRecord.push_back(vertexStack[vertexStack.size()-1]);
		QPointF vertex=vertexStack[vertexStack.size()-1].m_pos;
		vertexStack.pop_back();
		angleRecord.push_back(angleStack[angleStack.size()-1]);
		QPointF angle=angleStack[angleStack.size()-1];
		angleStack.pop_back();
		
		//skeleton::Edge m_edge=tracking(pos.x(),pos.y(),angle.x(),angle.y(), m_imgSize.width(), m_imgSize.height(), 4, pGradS,vertexStack,angleStack, vertexRecord, angleRecord);

		float seedx=vertex.x();
		float seedy=vertex.y();
		float tangx=angle.x();
		float tangy=angle.y();
		skeleton::Edge pos;//(QPointF(px,py));
		if(seedx<0 || seedy<0 || seedx>=w || seedy>=h)
		{
			cout << "seed should not out of the image." << endl;
			continue;
		}

		float prevTangx = tangx;
		float prevTangy = tangy;
		float stepPos[8];
		const int proWidth = 8;

		////first step: get out of the junction area, Ӧȸĳһ
		float direction[10];
		direction[0]=tangx;
		direction[1]=tangy;
		if (!ImagePaint::trackingEdge(pos,w,h,direction,seedx,seedy,cn,flow))///meet a junction point
		//seedx,seedy,startx,startyĸķ
		{//record the two possible vectors
			//junctionԼջ
			///////////////////////////////////һÿjunctionΧ
		    if (pos.m_vPoint.size()>3)
			{
				int pointSz=pos.m_vPoint.size();
				QPointF m_jPoint=pos.m_vPoint[pointSz-1];//Ǹjunction point
				
				///record¼ѾٹģstackǴtrack
				///յͷѾٹغϣ˵һȦһ򲢲¼
				///յͷtrackغϣһ·ڣ򲢲¼
				///Ѿٹ෴Ҳһ⣬Ȳ
				///ʹtrack෴ԭķɾ,򲻼¼
				int k=0;
				while(direction[2*k]!=0)
				{
					int cases=0;
					tangx=direction[2*k];
					tangy=direction[2*k+1];
					for (int i=0;i<vertexRecord.size();i++)
					{
						if (Dist(m_jPoint.x(),m_jPoint.y(),vertexRecord[i].m_pos.x(),vertexRecord[i].m_pos.y())<=10)
						{
							pos.m_vPoint.push_back(vertexRecord[i].m_pos);
							if ((tangx*angleRecord[i].x()+tangy*angleRecord[i].y())/Dist(tangx,tangy,0,0)/Dist(angleRecord[i].x(),angleRecord[i].y(),0,0)>cos(3.1415/12))
							{
								cases=2;
								break;
							}
							if ((tangx*angleRecord[i].x()+tangy*angleRecord[i].y())/Dist(tangx,tangy,0,0)/Dist(angleRecord[i].x(),angleRecord[i].y(),0,0)<-cos(3.1415/12))
							{
								cases=4;
								break;
							}
						}
					}
					for (int i=0;i<vertexStack.size();i++)
					{
						if (Dist(m_jPoint.x(),m_jPoint.y(),vertexStack[i].m_pos.x(),vertexStack[i].m_pos.y())<=10)
						{
							pos.m_vPoint.push_back(vertexStack[i].m_pos);
							if ((tangx*angleStack[i].x()+tangy*angleStack[i].y())/Dist(tangx,tangy,0,0)/Dist(angleStack[i].x(),angleStack[i].y(),0,0)>cos(3.1415/12))
							{
								cases=1;
								break;
							}
							if ((tangx*angleStack[i].x()+tangy*angleStack[i].y())/Dist(tangx,tangy,0,0)/Dist(angleStack[i].x(),angleStack[i].y(),0,0)<-cos(3.1415/12))
							{
								cases=3;
								break;
							}
						}
					}
					if ((cases!=1)&&(cases!=2)&&(cases!=4))//Ҫ¼
					{
						vertexStack.push_back(skeleton::Vertex::Vertex(QPointF(pos.m_vPoint[pos.m_vPoint.size()-1].x(),pos.m_vPoint[pos.m_vPoint.size()-1].y()),true));
						angleStack.push_back(QPointF(tangx,tangy));
						cout<<vertexStack[vertexStack.size()-1].m_pos.x()<<','<<vertexStack[vertexStack.size()-1].m_pos.y()<<endl;
						cout<<angleStack[angleStack.size()-1].x()<<' '<<angleStack[angleStack.size()-1].y()<<endl;
					}
					k++;
				}
				pointSz=pos.m_vPoint.size();
				m_jPoint=pos.m_vPoint[pointSz-1];//Ǹjunction point
				int index1=sktGraph.indexVertex(pos.m_vPoint[0]);
				int index2=sktGraph.indexVertex(m_jPoint);
				if (index2==-1)
				{
					sktGraph.m_vVertex.push_back(skeleton::Vertex::Vertex(m_jPoint,true));
					index2=sktGraph.m_vVertex.size()-1;
				}else
				{
					pos.m_vPoint[pointSz-1]=sktGraph.m_vVertex[index2].m_pos;
				}
				sktGraph.m_vVertex42edge.push_back(index1);
				sktGraph.m_vVertex42edge.push_back(index2);
				sktGraph.m_vEdge.push_back(pos);
		
			}
		}
		else if (pos.m_vPoint.size()>3)
		{
			sktGraph.m_vEdge.push_back(pos);
			int edgeSz=sktGraph.m_vEdge.size();
			int pointSz=sktGraph.m_vEdge[edgeSz-1].m_vPoint.size();
			QPointF m_jPoint=sktGraph.m_vEdge[edgeSz-1].m_vPoint[pointSz-1];
			sktGraph.m_vVertex.push_back(skeleton::Vertex::Vertex(m_jPoint,false));
		}	
	}
}

/*
bool leftWay(vector<tracking::Pot3f> &leftPos, vector<tracking::Pot3f> &rightPos)
{
	float left=0;
	float right=0;
	for (int i=1;i<leftPos.size();i++)
	{
		if (abs(leftPos[i].ang-leftPos[i-1].ang)>3)
		   left+=abs(-3.1415*2+abs(leftPos[i].ang-leftPos[i-1].ang));
		else
    		left+=abs(leftPos[i].ang-leftPos[i-1].ang);
		if (abs(rightPos[i].ang-rightPos[i-1].ang)>3)
			right+=abs(-3.1415*2+abs(rightPos[i].ang-rightPos[i-1].ang));
		else
    		right+=abs(rightPos[i].ang-rightPos[i-1].ang);
	}
	if (left<right)
		return true;
	return false;
}
*/
void ImagePaint::Record(skeleton::Edge &pos, float width, int w, int h)
{
	if (width>8) width=8;
	if (pos.m_vPoint.size()>3)
	for (int i=1;i<pos.m_vPoint.size()-1;i++)
	{
		float px=pos.m_vPoint[i].x();
		float py=pos.m_vPoint[i].y();
		for (int xx=int(-width/2-0.5);xx<width/2+0.6;xx++)
			for (int yy=int(-width/2-0.5);yy<width/2+0.6;yy++)
				if (xx*xx+yy*yy<=width*width/4 && isInImage(int(px+xx+0.5),int(py+yy+0.5),w,h))
				{
					m_record.setPixel(int(px+xx+0.5),int(py+yy+0.5),0);
				}
	}
}

int ImagePaint::dirJunction(float *direction, int seedx,int seedy,int w,int h,int width,float prevTangx,float prevTangy)
{
	int red100[100];
	int k=0;
	//width=Dist(leftPos[leftPos.size()/2].x,leftPos[leftPos.size()/2].y,rightPos[rightPos.size()/2].x,rightPos[rightPos.size()/2].y);
	k=0;
	for (int i=0;i<100;i++)///iӦĽǶi/100*2*pi
	{
		float tmpx=seedx+cos(2*3.14*i/100)*(width+3);
		float tmpy=seedy+sin(2*3.14*i/100)*(width+3);
		if (!isInImage(tmpx,tmpy,w,h))
		{
			red100[i]=0;
			continue;
		}
		red100[i]=qRed(m_image.pixel(tmpx,tmpy));
	}
	while(k<=3)
	{
		//ĵ;
		float maxRed=0;
		float maxI=0;
		for (int i=0;i<100;i++)
		{
			if (red100[i]>maxRed)
			{
				maxRed=red100[i];
				maxI=i;
			}
		}
		float tangx=cos(maxI*2*3.1415/100);
		float tangy=sin(maxI*2*3.1415/100);
		if (maxRed<50)
		{
			break;
		}
		if (-tangx*prevTangx-tangy*prevTangy<=cos(3.14/12))
		{
			direction[k*2]=tangx;
			direction[k*2+1]=tangy;
			k++;
		}
		int tmpI=maxI;
		while(red100[tmpI]>40)
		{
			red100[tmpI]=0;
			tmpI--;
			if (tmpI<0) tmpI=99;
		}
		tmpI=maxI+1;
		if (tmpI==100)	tmpI=0;
		while(red100[tmpI]>40)
		{
			red100[tmpI]=0;
			tmpI++;
			if (tmpI==100) tmpI=0;
		}
	}
	direction[2*k]=0;
	direction[2*k+1]=0;
	return k;
}

bool ImagePaint::trackingEdge(skeleton::Edge &pos, int w, int h, float *direction, float px, float py, int cn, float *flow)
{
	//skeleton::Edge pos(QPointF(px,py));
	if(px<0 || py<0 || px>=w || py>=h)
	{
		cout << "seed should not out of the image." << endl;
		//return pos;
	}
	float tangx=direction[0];
	float tangy=direction[1];
	if (qRed(m_record.pixel(int(0.5+px+tangx*10),int(0.5+py+tangy*10)))==0)
	{
		return true;
	}

	int cnt = 0;
	float seedx = float(px);
	float seedy = float(py);
	//const float *fcenter = getDirection(w,h,cn,flow,px,py);
	float stepPos[8];
 	float dist=sqrt(tangx*tangx+tangy*tangy);
	tangx=tangx/dist;
	tangy=tangy/dist;
	float fleftx,flefty,frightx,frighty;
	float width=0;
	pos.m_vPoint.push_back(QPointF(float(px),float(py)));
	seedx+=tangx*10;
	seedy+=tangy*10;
	pos.m_vPoint.push_back(QPointF(seedx,seedy));//ֱ߳
	//cout<<seedx<<','<<seedy<<endl;         
	///ʼһν߽ϢҲݶϢһʼjunctionܻ
	float prevTangx=tangx;
	float prevTangy=tangy;
	while(cnt<1 || judgeJunctionArea(fleftx,flefty,frightx,frighty) || judgeJunctionArea(stepPos[0],stepPos[1],stepPos[4],stepPos[5],seedx,seedy))//ǰ4أһЩboundaryжϣ߳junctionArea
	{
		const float *fseed0 = getDirection(w,h,4,flow,int(seedx+0.5),int(seedy+0.5));
		tangx=fseed0[1];
		tangy=-fseed0[0];
		float dist=sqrt(tangx*tangx+tangy*tangy);
		tangx=tangx/dist;
		tangy=tangy/dist;
		//ԭһ£Ҫ㷴
		float tangsign = tangx*prevTangx + tangy*prevTangy;
		if(tangsign<0.0)
		{
			tangx = -tangx;
			tangy= -tangy;
		}		
		seedx += tangx; //tangx>0.5?1: (tangx<-0.5?-1:0);
		seedy += tangy; //tangy>0.5?1: (tangy<-0.5?-1:0);
		prevTangx=tangx;
		prevTangy=tangy;
		if (!findBoundary(stepPos,seedx,seedy,w,h,fseed0[1],-fseed0[0],8,flow,m_image))//ֱǰûм¼
		{
			//seedx=(stepPos[0]+stepPos[4])/2;
			//seedy=(stepPos[1]+stepPos[5])/2;
			//pos.m_vPoint.push_back(QPointF(seedx,seedy));
			continue;
		}
		//ұ߽ǶԵˣʹñ߽ϵķ
	    const float *fseed = getDirection(w,h,4,flow,int(stepPos[0]+0.5),int(stepPos[1]+0.5));
		fleftx = fseed[1];
		flefty = -fseed[0];
		if (fleftx*tangx+flefty*tangy<0)
		{
			fleftx=-fleftx;
			flefty=-flefty;
		}
		const float *fseed2 = getDirection(w,h,4,flow,int(stepPos[4]+0.5),int(stepPos[5]+0.5));
		frightx = fseed2[1];
		frighty = -fseed2[0];
		if (frightx*tangx+frighty*tangy<0)
		{
			frightx=-frightx;
			frighty=-frighty;
		}
		pos.m_vPoint.push_back(QPointF(float(seedx),float(seedy)));
		cnt ++;

		//жǷĩ
		if  (qRed(m_record.pixel(int(seedx+0.5),int(seedy+0.5))==0)||!isInImage(seedx+tangx,seedy+tangy,w,h))
		{
			Record(pos,Dist(stepPos[0],stepPos[1],stepPos[4],stepPos[5]),w,h);
			return true;//Ƿֲ棬ҪߣҲҪJunctionջ
		}
	}
	///ǰһֵ߳junctionΧһпܳ
	//ҳϢ
	float prevx = tangx;
	float prevy = tangy;
	//const float *fseed = getDirection(w,h,4,flow,int(seedx+0.5),int(seedy+0.5));
	vector<tracking::Pot3f> leftPos, rightPos;
	
	float leftx=stepPos[0];
	float lefty=stepPos[1];
	float rightx=stepPos[4];
	float righty=stepPos[5];
	///ʱҽϢǿġǰƵأǰ
	seedx=(leftx+rightx)/2.0;
	seedy=(lefty+righty)/2.0;
	pos.m_vPoint.push_back(QPointF(seedx,seedy));
	float angle=atan2(tangx,tangy);
	leftPos.push_back(tracking::Pot3f(leftx,lefty,angle));
	rightPos.push_back(tracking::Pot3f(rightx,righty,angle));
	width=Dist(leftx,lefty,rightx,righty);
	int count=0;
	bool flag=false;
	int recordCnt;
	cnt=0;
	float prevmidx=tangx;
	float prevmidy=tangy;
	while(cnt<300)
	{
		///
		const float *fseed = getDirection(w,h,4,flow,int(leftx+0.5),int(lefty+0.5));
		tangx = fseed[1];
		tangy = -fseed[0];
		//һ֪Ϊʲôֺö඼ûйһ
		float dist=sqrt(tangx*tangx+tangy*tangy);
		tangx=tangx/dist;
		tangy=tangy/dist;
		//ԭһ£Ҫ㷴
		float tangsign = tangx*prevTangx + tangy*prevTangy;
		if(tangsign<0.0)
		{
			tangx = -tangx;
			tangy= -tangy;
		}		
		
		///ǰһѰλ
		leftx += tangx;
		lefty += tangy;
		prevTangx = tangx;
		prevTangy = tangy;
				
		//һҲ֮ǰ
		const float *f = getDirection(w,h,cn,flow,int(rightx+0.5),int(righty+0.5));
		tangx=f[1];
		tangy=-f[0];
	 	if (tangx*prevx+tangy*prevy<0)
		{
			tangx=-tangx;
			tangy=-tangy;
		}
		prevx=tangx;prevy=tangy;
		rightx+=tangx;
		righty+=tangy;
		/////////////////////////////////////////ұ߽һˣ
		if (Dist(leftx,lefty,rightx,righty)<2|| !flag && judgeJunctionArea(leftx,lefty,rightx,righty,seedx,seedy))
		{
			const float *fseed = getDirection(w,h,4,flow,int(leftx+0.5),int(lefty+0.5));
			tangx = fseed[1];
			tangy = -fseed[0];
			//һ֪Ϊʲôֺö඼ûйһ
			float dist=sqrt(tangx*tangx+tangy*tangy);
			tangx=tangx/dist;
			tangy=tangy/dist;
			//ԭһ£Ҫ㷴
			float tangsign = tangx*prevTangx + tangy*prevTangy;
			if(tangsign<0.0)
			{
	    		tangx = -tangx;
				tangy= -tangy;
			}		
			findBoundary(stepPos,(leftx+rightx)/2,(lefty+righty)/2,w,h,tangx,tangy,8,flow,m_image);
			if (Dist(leftx,lefty,stepPos[0],stepPos[1])<Dist(rightx,righty,stepPos[0],stepPos[1]))
			{
				leftx=stepPos[0];
				lefty=stepPos[1];
				rightx=stepPos[4];
				righty=stepPos[5];
			}else
			{
				rightx=stepPos[0];
				righty=stepPos[1];
				leftx=stepPos[4];
				lefty=stepPos[5];
			}
			width=Dist(leftx,lefty,rightx,righty);
		}
		///////////////////////////////
		/////ʹ߱ͬ
		for(int i=1;i<3;i++)
		{
			const float *fseed = getDirection(w,h,4,flow,int(leftx+0.5),int(lefty+0.5));
			tangx = fseed[1];
			tangy = -fseed[0];
			//һ֪Ϊʲôֺö඼ûйһ
			float dist=sqrt(tangx*tangx+tangy*tangy);
			tangx=tangx/dist;
			tangy=tangy/dist;
			//ԭһ£Ҫ㷴
			float tangsign = tangx*prevTangx + tangy*prevTangy;
			if(tangsign<0.0)
			{
				tangx = -tangx;
				tangy= -tangy;
			}
			if (Dist(leftx+tangx,lefty+tangy,rightx,righty)<Dist(leftx,lefty,rightx,righty))
			{
				leftx+=tangx;
			    lefty+=tangy;
				prevTangx = tangx;
				prevTangy = tangy;			
			}else
			{
				const float *f = getDirection(w,h,cn,flow,int(rightx+0.5),int(righty+0.5));
				tangx=f[1];
				tangy=-f[0];
	 			if (tangx*prevx+tangy*prevy<0)
				{
					tangx=-tangx;
					tangy=-tangy;
				}
				if (Dist(rightx+tangx,righty+tangy,leftx,lefty)<Dist(rightx,righty,leftx,lefty))
				{
					prevx=tangx;prevy=tangy;
					rightx+=tangx;
					righty+=tangy;
				}else///ҶƶˣǰλѣԿ
					break;

			}

		}
		float angle1=atan2(prevTangy,prevTangx);
		float angle2=atan2(prevx,prevy);
		leftPos.push_back(tracking::Pot3f(leftx,lefty,angle1));
		rightPos.push_back(tracking::Pot3f(rightx,righty,angle2));

		//////////////ҵͬˣұ߽ľжǷֿ
		
		///жǷjunction
		cnt++;
		if (!flag)
		{
			//seedx=(leftx+rightx)/2;
			//seedy=(lefty+righty)/2;
			const float *f = getDirection(w,h,cn,flow,int(seedx+0.5),int(seedy+0.5));
			tangx=f[1];
			tangy=-f[0];
		 	if (tangx*prevmidx+tangy*prevmidy<0)
			{
				tangx=-tangx;
				tangy=-tangy;
			}
			prevmidx=tangx;prevmidy=tangy;
			seedx+=tangx;
			seedy+=tangy;
			
			for(int i=1;i<3;i++)
			{
				const float *fseed = getDirection(w,h,4,flow,int(seedx+0.5),int(seedy+0.5));
				tangx = fseed[1];
				tangy = -fseed[0];
				//һ֪Ϊʲôֺö඼ûйһ
				float dist=sqrt(tangx*tangx+tangy*tangy);
				tangx=tangx/dist;
				tangy=tangy/dist;
				//ԭһ£Ҫ㷴
				float tangsign = tangx*prevmidx + tangy*prevmidy;
				if(tangsign<0.0)
				{
					tangx = -tangx;
					tangy= -tangy;
				}
				if (Dist(seedx+tangx,seedy+tangy,rightx,righty)+Dist(seedx+tangx,seedy+tangy,leftx,lefty)<Dist(leftx,lefty,seedx,seedy)+Dist(rightx,righty,seedx,seedy))
				{
					seedx+=tangx;
				    seedy+=tangy;
					prevmidx = tangx;
					prevmidy = tangy;			
				}else
					break;
			}
			pos.m_vPoint.push_back(QPointF(seedx,seedy));
			//cout<<leftx<<','<<lefty<<"    "<<(leftx+rightx)/2<<','<<(lefty+righty)/2<<"      "<<rightx<<','<<righty<<endl;
		}
		if (!flag && cnt>3)
		{
			////һǵ߶ĩβˣһ߷ֿ
			float fleftx=(leftPos[leftPos.size()-1].x-leftPos[leftPos.size()-4].x)/3;
			float flefty=(leftPos[leftPos.size()-1].y-leftPos[leftPos.size()-4].y)/3;
			float frightx=(rightPos[rightPos.size()-1].x-rightPos[rightPos.size()-4].x)/3;
			float frighty=(rightPos[rightPos.size()-1].y-rightPos[rightPos.size()-4].y)/3;
			if (judgeJunctionArea(fleftx,flefty,frightx,frighty))///߷ֿ30,smooth=8ӦÿԵ
			{
				flag=true;
				recordCnt=cnt;
				//continue;
			}
			width=Dist(leftx,lefty,rightx,righty);///Щ仯Ҫʱ
			if (judgeEdgeEnd(seedx,seedy,prevTangx,prevTangy,w,h,3,m_record) && judgeEdgeEnd(seedx,seedy,prevx,prevy,w,h,3,m_record))
			{
				///һ䲻ܷ棬ڷֲ·ҲᱻжϳedgeEnd
				Record(pos,width,w,h);
				return true;
			}
		}
		///֮Ѿֹˣ
		if (flag && Dist(leftx,lefty,rightx,righty)>2*width && Dist(leftx,lefty,rightx,righty>12) && cnt-recordCnt>15)///ֿһûģ淽Ǵӿʼб仯֮ٿʼ
		{
			break;
		}
		if (!isInImage(leftx+2*prevTangx,lefty+2*prevTangy,w,h) || !isInImage(rightx+2*prevx,righty+2*prevy,w,h))
		{
			if (!flag)//ûJunctionˣ˵ĩβ
			{
				Record(pos,width,w,h);
				return true;
			}else//ѾjunctionҪȥѰҺJunctionλ
			{
				break;
			}
		}
		if (!isInImage((leftx+rightx)/2+2*tangx,(lefty+righty)/2+2*tangy,w,h))
		{
			Record(pos,width,w,h);
			return true;
		}
		if (cnt>=4 && qRed(m_record.pixel(pos.m_vPoint[pos.m_vPoint.size()-5].x(),pos.m_vPoint[pos.m_vPoint.size()-5].y()))==0)
		{
			Record(pos,width,w,h);
			return true;
		}
	}
	
	///ӿʼпֲܷǰ5ؼ㿪ʼѡһʵλΪؼ
	seedx=(leftPos[recordCnt].x+rightPos[recordCnt].x)/2;
	seedy=(leftPos[recordCnt].y+rightPos[recordCnt].y)/2;
	prevTangx=pos.m_vPoint[pos.m_vPoint.size()-1].x()-pos.m_vPoint[pos.m_vPoint.size()-2].x();
	prevTangy=pos.m_vPoint[pos.m_vPoint.size()-1].y()-pos.m_vPoint[pos.m_vPoint.size()-2].y();
	for(int i=recordCnt+1;i<cnt;i++)
	{
		const float *fseed = getDirection(w,h,4,flow,int(seedx+0.5),int(seedy+0.5));
		tangx = fseed[1];
		tangy = -fseed[0];
		//һ֪Ϊʲôֺö඼ûйһ
		float dist=sqrt(tangx*tangx+tangy*tangy);
		tangx=tangx/dist;
		tangy=tangy/dist;
		//ԭһ£Ҫ㷴
		float tangsign = tangx*prevTangx + tangy*prevTangy;
		if(tangsign<0.0)
		{
			tangx = -tangx;
			tangy = -tangy;
		}
		seedx+=tangx;
		seedy+=tangy;
		prevTangx=tangx;
		prevTangy=tangy;
		
		
		if (Dist(seedx,seedy,(leftPos[i].x+rightPos[i].x)/2,(leftPos[i].y+rightPos[i].y)/2)>2)
		{
			//seedx-=tangx;
			//seedy-=tangy;
			break;///ҵһȽϺJunction
		}
		cout<<i<<"   "<<seedx<<','<<seedy<<endl;
	}
	cout<<cnt<<endl<<endl;
	///junctionһԲԲ߽ϵ
	int red100[100];
	
	int k=0;
	//width=Dist(leftPos[leftPos.size()/2].x,leftPos[leftPos.size()/2].y,rightPos[rightPos.size()/2].x,rightPos[rightPos.size()/2].y);
	while(k<2 && width<12)
	{
		k=dirJunction(direction, seedx,seedy,w,h, width, prevTangx, prevTangy);
		width+=1;
	}	
	float position[10];
	width-=1;
	for (int i=0;i<k;i++)
	{
		position[2*i]=seedx+direction[2*i]*(width+3);
		position[2*i+1]=seedy+direction[2*i+1]*(width+3);
	}

	////Ѽ·λöҺˣjunctionģǰķ
	float maxRed=0;
	float tmpx=pos.m_vPoint[pos.m_vPoint.size()-1].x();
	float tmpy=pos.m_vPoint[pos.m_vPoint.size()-1].y();
	float recordx=0;
	float recordy=0;
	for(int i=recordCnt;i<cnt;i++)
	{
		const float *fseed = getDirection(w,h,4,flow,int(tmpx+0.5),int(tmpy+0.5));
		tangx = fseed[1];
		tangy = -fseed[0];
		//һ֪Ϊʲôֺö඼ûйһ
		float dist=sqrt(tangx*tangx+tangy*tangy);
		tangx=tangx/dist;
		tangy=tangy/dist;
		//ԭһ£Ҫ㷴
		float tangsign = tangx*prevTangx + tangy*prevTangy;
		if(tangsign<0.0)
		{
			tangx = -tangx;
			tangy = -tangy;
		}
		tmpx+=tangx;
		tmpy+=tangy;
		prevTangx=tangx;
		prevTangy=tangy;
		float red=0;
		for (int j=0;j<k;j++)
		{
			float xx=position[j*2];
			float yy=position[j*2+1];
			red+=qRed(m_image.pixel((tmpx+3*xx)/4,(tmpy+3*yy)/4))+qRed(m_image.pixel((tmpx+xx)/2,(tmpy+yy)/2));
			red+=qRed(m_image.pixel((tmpx*3+xx)/4,(tmpy*3+yy)/4))+qRed(m_image.pixel(tmpx,tmpy));
		}
		if (red>maxRed)
		{
			maxRed=red;
			seedx=tmpx;
			seedy=tmpy;
			recordx=tangx;
			recordy=tangy;
			pos.m_vPoint.push_back(QPointF(seedx,seedy));
		}
	}
	//pos.m_vPoint.push_back(QPointF(seedx,seedy));
	
	///¶λ
	
	//int red100[100];
	k=0;
	for (int i=0;i<100;i++)///iӦĽǶi/100*2*pi
	{
		float tmpx=seedx+cos(2*3.14*i/100)*(width+3);
		float tmpy=seedy+sin(2*3.14*i/100)*(width+3);
		if (!isInImage(tmpx,tmpy,w,h))
		{
			red100[i]=0;
			continue;
		}
		red100[i]=qRed(m_image.pixel(tmpx,tmpy));
	}
	while(k<=3)
	{
		//ĵ;
	 	float maxRed=0;
		float maxI=0;
		for (int i=0;i<100;i++)
	 	{
			if (red100[i]>maxRed)
			{
				maxRed=red100[i];
				maxI=i;
			}
		}
		float tangx=cos(maxI*2*3.1415/100);
		float tangy=sin(maxI*2*3.1415/100);
		if (maxRed<50)
		{
			break;
		}
		if (-tangx*recordx-tangy*recordy<=cos(3.14/12)&&m_record.pixel(seedx+tangx*width,seedy+tangy*width)>20)
		if (m_record.pixel(seedx+tangx*width/3,seedy+tangy*width/3)>20 && m_record.pixel(seedx+tangx*2*width/3,seedy+tangy*2*width/3)>20)
		{
			direction[k*2]=tangx;
			direction[k*2+1]=tangy;
			k++;
		}
		int tmpI=maxI;
		while(red100[tmpI]>40)
		{
			red100[tmpI]=0;
			tmpI--;
			if (tmpI<0) tmpI=99;
		}
		tmpI=maxI+1;
		if (tmpI==100)	tmpI=0;
		while(red100[tmpI]>40)
		{
			red100[tmpI]=0;
			tmpI++;
			if (tmpI==100) tmpI=0;
		}
	}
	direction[2*k]=0;
	direction[2*k+1]=0;
	width=Dist(leftPos[leftPos.size()/2].x,leftPos[leftPos.size()/2].y,rightPos[rightPos.size()/2].x,rightPos[rightPos.size()/2].y);
	//¼
	Record(pos,width,w,h);
	return false;
}
