
#include "vectorizationWindow.h"
#include "param.h"
#include "paramui.h"

MainWindow *MainWindow::_mainWindow = NULL;

MainWindow::MainWindow() {
    setupUi(this);
	branchDockWidget->setFloating(true);
	branchDockWidget->setVisible(false);
	connect(branchPushButton,SIGNAL(clicked()),m_imagePaint,SLOT(branchPushButtonClicked()));

    m_dirty = false;
	addAction(actionQuit);
	addAction(actionToolBox);
	addAction(actionFullScreen);
	addAction(actionReloadShader);
	addAction(actionSnapshot);
	addAction(actionMove_Vertex);
	addAction(actionDelete_Vertex);
	addAction(actionSplit_Vertex);
	addAction(actionDraw_Edge);
	addAction(actionLoad_Skeleton);
	addAction(actionSave_Skeleton);
	addAction(actionLoad_SVG);
	addAction(actionSave_SVG);
	addAction(actionLoad_Parameter);
	addAction(actionSave_Parameter);
	addAction(actionSave_Everything);

	m_interactionToolBar = addToolBar(tr("interaction"));
	m_interactionToolBar->addAction(actionMove_Vertex);
	m_interactionToolBar->addAction(actionDelete_Vertex);
	m_interactionToolBar->addAction(actionSplit_Vertex);
	m_interactionToolBar->addAction(actionDraw_Edge);
	connect(actionMove_Vertex,SIGNAL(triggered(bool)),this,SLOT(setMoveMode(bool)));
	connect(actionDelete_Vertex,SIGNAL(triggered(bool)),this,SLOT(setDeleteMode(bool)));
	connect(actionSplit_Vertex,SIGNAL(triggered(bool)),this,SLOT(setSplitMode(bool)));
	connect(actionDraw_Edge,SIGNAL(triggered(bool)),this,SLOT(setDrawMode(bool)));

	connect(actionLoad_Skeleton,SIGNAL(triggered()),m_imagePaint,SLOT(loadSkeletonFile()));
	connect(actionSave_Skeleton,SIGNAL(triggered()),m_imagePaint,SLOT(saveSkeletonFile()));
	connect(actionLoad_SVG,SIGNAL(triggered()),m_imagePaint,SLOT(loadSVGFile()));
	connect(actionSave_SVG,SIGNAL(triggered()),m_imagePaint,SLOT(saveSVGFile()));
	connect(actionLoad_Parameter,SIGNAL(triggered()),m_imagePaint,SLOT(loadParameterFile()));
	connect(actionSave_Parameter,SIGNAL(triggered()),m_imagePaint,SLOT(saveParameterFile()));
	connect(actionSave_Everything,SIGNAL(triggered()),m_imagePaint,SLOT(saveEverythingFile()));

	m_dockWidget->move(0,0);

    m_imagePaint->setFocus();

	m_selectitem->addItems(displayList);
	m_selectitem->setCurrentIndex(0);

	//////////////////////// set up the parameter part of the toolbox 
	/// add all the parameter units
	new ParamInt(this, "gradSize", 5, 1, 100, 1, m_imagePaint->gradSizePtr());
	new ParamInt(this, "gradSmoothSize", 5, 1, 100, 1, m_imagePaint->gradSmoothSizePtr());
	new ParamInt(this, "maxStepWidth", 8, 3, 20, 1, m_imagePaint->maxStepWidthPtr());
	new ParamInt(this, "minLuminanceThre", 90, 20, 250, 1, m_imagePaint->minLuminanceThrePtr());
	new ParamInt(this, "angleJunctionThre", 30, 1, 90, 1, m_imagePaint->angleJunctionThrePtr());
	new ParamInt(this, "angleSmoothThre", 150, 1, 180, 1, m_imagePaint->angleSmoothThrePtr());

	new ParamDouble(this, "widthf", 0.25, 0.0, 1.0, 0.01, m_imagePaint->widthfPtr());
	new ParamInt(this, "trackingLength", 40, 5, 200, 5, m_imagePaint->trackingLengthPtr());

	new ParamInt(this, "signRadius", 8, 1, 30, 1, m_imagePaint->signRadiusPtr());
	new ParamInt(this, "signSection", 16, 2, 100, 1, m_imagePaint->signSectionPtr());

	QStringList paramList;
	paramList << QString("gradSize") << QString("gradSmoothSize") 
		<< QString("maxStepWidth") << QString("minLuminanceThre") 
		<< QString("angleJunctionThre") << QString("angleSmoothThre") 
		<< QString("widthf") << QString("trackingLength")
		<< QString("signRadius") << QString("signSection");
	ParamUI *pui = new ParamUI(this, this, paramList);
	pui->setMinimumWidth(100);
	pui->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
	m_vboxTool->addWidget(pui);

	QComboBox *typeComboBox = new QComboBox();
	typeComboBox->addItem(QString("StreamlineEuler Nearest"));
	typeComboBox->addItem(QString("Streamline: Euler Bilinear"));
	typeComboBox->addItem(QString("Streamline: Runge-Kutta Nearest"));
	typeComboBox->addItem(QString("Streamline: Runge-Kutta Bilinear"));
	typeComboBox->addItem(QString("Streamline: Runge-Kutta Step"));
	typeComboBox->addItem(QString("Streamline: Runge-Kutta Profile"));
	typeComboBox->addItem(QString("Contour Streamline"));
	typeComboBox->setCurrentIndex(0);
	connect(typeComboBox, SIGNAL(currentIndexChanged(int)), m_imagePaint, SLOT(setStreamlineType(int)));
	m_vboxTool->addWidget(typeComboBox);

	QPushButton *sl = new QPushButton(QString("StreamLine"),this);
	sl->setCheckable(true);
	sl->setChecked(false);
	connect(sl, SIGNAL(clicked(bool)), m_imagePaint, SLOT(setStreamline(bool)));
	m_vboxTool->addWidget(sl);

	m_computeSkeletonPB = new QPushButton(QString("Compute Skeleton"),this);
	m_computeSkeletonPB->setCheckable(true);
	m_computeSkeletonPB->setChecked(false);
	connect(m_computeSkeletonPB, SIGNAL(clicked(bool)), m_imagePaint, SLOT(ComputeSkeletonChicked(bool)));
	m_vboxTool->addWidget(m_computeSkeletonPB);	

	QPushButton *arrowBtn = new QPushButton(QString("Draw Arrow Field"),this);
	arrowBtn->setCheckable(true);
	arrowBtn->setChecked(false);
	connect(arrowBtn, SIGNAL(clicked(bool)), m_imagePaint, SLOT(setDrawArrowField(bool)));
	m_vboxTool->addWidget(arrowBtn);	

	m_profileLabel = new QLabel(this);
	m_profileLabel->setFixedSize(200,200);
	m_profileLabel->setFrameShape(QFrame::Box);

	//QImage img("F:\\data\\image\\baboon.PNG");
	//m_profileLabel->setPixmap(QPixmap::fromImage(img));

	m_vboxTool->addWidget(m_profileLabel);
	m_vboxTool->addStretch(100);
	
	/// examples
	//new ParamGroup(this, "T1", true);
	//new ParamBool(this, "T2", true);
	//new ParamString(this, "T3", QString("whT3"), true);
	//new ParamChoice(this, "T4", QString("wsT4"), QString("ws2T4|ws2T41|ws2T42|ws2T43"));
	//new ParamEnum(this, "T5", 3, QString("wssT5|wssT51|wssT52"));
	//new ParamImage(this, "T6", QString("test.png"));
    //new ParamInt   (this, "N", 5, 1, 100, 1, &m_N);
    //new ParamDouble(this, "sigma_d", 1.0, 0.0, 10.0, 0.05, &m_sigma_d);

	/////////////////////////////// connection building
    connect(m_selectitem, SIGNAL(currentIndexChanged(int)), m_imagePaint, SLOT(onIndexChanged(int)));

    m_player = new VideoPlayer(this, ":/test.png");
    connect(m_player, SIGNAL(videoChanged(int)), this, SLOT(onVideoChanged(int)));
    connect(m_player, SIGNAL(currentFrameChanged(int)), this, SLOT(setDirty()));
    connect(m_player, SIGNAL(outputChanged(const QImage&)), m_imagePaint, SLOT(setImage(const QImage&)));
    connect(this, SIGNAL(imageChanged(const QImage&)), m_player, SLOT(setOutput(const QImage&)));

    m_videoControls->setFrameStyle(QFrame::NoFrame);
    m_videoControls->setAutoHide(true);
    connect(m_videoControls, SIGNAL(stepForward()), m_player, SLOT(stepForward()));
    connect(m_videoControls, SIGNAL(stepBack()), m_player, SLOT(stepBack()));
    connect(m_videoControls, SIGNAL(currentFrameTracked(int)), m_player, SLOT(setCurrentFrame(int)));
    connect(m_videoControls, SIGNAL(playbackChanged(bool)), m_player, SLOT(setPlayback(bool)));
    connect(m_videoControls, SIGNAL(trackingChanged(bool)), this, SLOT(setDirty()));

    connect(m_player, SIGNAL(videoChanged(int)), m_videoControls, SLOT(setFrameCount(int)));
    connect(m_player, SIGNAL(playbackChanged(bool)), m_videoControls, SLOT(setPlayback(bool)));
    connect(m_player, SIGNAL(currentFrameChanged(int)), m_videoControls, SLOT(setCurrentFrame(int)));

	/// status bar
	statusBar()->showMessage(tr("WElCOME!"));

	m_statusMouseLabel = new QLabel(this);
	statusBar()->addPermanentWidget(m_statusMouseLabel);
	m_statusMouseLabel->setText("| Pos: (0,0)");

	m_statusZoomLabel = new QLabel(this);
	statusBar()->addPermanentWidget(m_statusZoomLabel);
	m_statusZoomLabel->setText("| Zoom: 100%");

	branchDockWidget->setVisible(false);
	branchDockWidget->hide();
}
void MainWindow::setMoveMode(bool isok)
{
	m_imagePaint->setInteractionMode(isok?MODE_MOVE:MODE_NONE);
	actionDelete_Vertex->setChecked(false);
	actionSplit_Vertex->setChecked(false);
	actionDraw_Edge->setChecked(false);
	QPixmap pixmap("move.png");
	QCursor cursor = QCursor(pixmap.scaled(20,20),1,1);
	setCursor(isok?cursor:Qt::OpenHandCursor);
	m_imagePaint->setCursor(isok?cursor:Qt::OpenHandCursor);
}
void MainWindow::setDeleteMode(bool isok)
{
	m_imagePaint->setInteractionMode(isok?MODE_DELETE:MODE_NONE);
	actionMove_Vertex->setChecked(false);
	actionSplit_Vertex->setChecked(false);
	actionDraw_Edge->setChecked(false);
	QPixmap pixmap("delete.png");
	QCursor cursor = QCursor(pixmap.scaled(30,30),1,1);
	setCursor(isok?cursor:Qt::OpenHandCursor);
	m_imagePaint->setCursor(isok?cursor:Qt::OpenHandCursor);
}
void MainWindow::setSplitMode(bool isok)
{
	m_imagePaint->setInteractionMode(isok?MODE_SPLIT:MODE_NONE);
	actionMove_Vertex->setChecked(false);
	actionDelete_Vertex->setChecked(false);
	actionDraw_Edge->setChecked(false);
	QPixmap pixmap("split.png");
	QCursor cursor = QCursor(pixmap.scaled(20,20),1,1);
	setCursor(isok?cursor:Qt::OpenHandCursor);
	m_imagePaint->setCursor(isok?cursor:Qt::OpenHandCursor);
}
void MainWindow::setDrawMode(bool isok)
{
	m_imagePaint->setInteractionMode(isok?MODE_DRAW:MODE_NONE);
	actionMove_Vertex->setChecked(false);
	actionDelete_Vertex->setChecked(false);
	actionSplit_Vertex->setChecked(false);
	QPixmap pixmap("draw.png");
	QCursor cursor = QCursor(pixmap.scaled(20,20),1,1);
	setCursor(isok?cursor:Qt::OpenHandCursor);
	m_imagePaint->setCursor(isok?cursor:Qt::OpenHandCursor);
}
void MainWindow::setProfileLabel(const QImage &img)
{
	if(img.width()==200 && img.height()==200)
		m_profileLabel->setPixmap(QPixmap::fromImage(img));
	else
		m_profileLabel->setPixmap(QPixmap::fromImage(img.scaled(200,200)));
}
void MainWindow::launchBranchWidget()
{
	branchDockWidget->setVisible(true);
	branchDockWidget->move(width()/2,height()/2);
}
MainWindow::~MainWindow() {
}

void MainWindow::restoreSettings() {
    QSettings settings;
    restoreGeometry(settings.value("mainWindow/geometry").toByteArray());
    restoreState(settings.value("mainWindow/windowState").toByteArray());

    settings.beginGroup("imagePaint");
    m_imagePaint->restoreSettings(settings);
    settings.endGroup();

    settings.beginGroup("parameters");
    AbstractParam::restoreSettings(settings, this);
    settings.endGroup();

    m_player->restoreSettings(settings);
}

void MainWindow::closeEvent(QCloseEvent *e) {
    QSettings settings;
    settings.setValue("mainWindow/geometry", saveGeometry());
    settings.setValue("mainWindow/windowState", saveState());

    settings.beginGroup("imageView");
    m_imagePaint->saveSettings(settings);
    settings.endGroup();

    settings.beginGroup("parameters");
    AbstractParam::saveSettings(settings, this);
    settings.endGroup();

    m_player->saveSettings(settings);

    QMainWindow::closeEvent(e);
} 

void MainWindow::on_actionOpen_triggered() {
	m_player->open();
}

void MainWindow::on_actionAbout_triggered() {
    QMessageBox msgBox;
    msgBox.setWindowTitle("About");
    msgBox.setIcon(QMessageBox::Information);
    msgBox.setText(
        "<html><body>" \
        "<p><b>Semantic-based LoD</b><br/><br/>" \
        "Author: Jiazhou CHEN <br/>" \
        "Date: " __DATE__ "</p>" \
        "<p>This program is free software: you can redistribute it and/or modify " \
        "it under the terms of the GNU General Public License as published by " \
        "the Free Software Foundation, either version 3 of the License, or " \
        "(at your option) any later version.</p>" \
        "<p>This program is distributed in the hope that it will be useful, " \
        "but WITHOUT ANY WARRANTY; without even the implied warranty of " \
        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the " \
        "GNU General Public License for more details.</p>" \

		"Parts of the framework of this software are copied and modified " \
		"from the source code of the softwares developed by: <br/>" \
		" Jan Eric Kyprianidis &lt;" \
		"<a href='http://www.kyprianidis.com'>www.kyprianidis.com</a> &gt; & <br/>" \
		" Romain Vergn &lt;" \
		"<a href='http://romain.vergne.free.fr'>http://romain.vergne.free.fr</a>&gt;<br/>" \
        /*"Related Publications:" \
        "<ul>" \
        "<li>" \
        "Kyprianidis, J. E., &amp; Kang, H. (2011). " \
        "Image and Video Abstraction by Coherence-Enhancing Filtering. " \
        "<em>Computer Graphics Forum</em>, 30(2), 593-602. " \
        "(Proceedings Eurographics 2011)" \
        "</li>" \
        "</ul>" \
        "<p>Test image courtesy of Ivan Mlinar @ flickr.com.</p>" \*/
        "</body></html>"
    );
    msgBox.setStandardButtons(QMessageBox::Ok);
    msgBox.exec();
}

void MainWindow::on_actionRecord_triggered() {
    m_player->record();
}

void MainWindow::setFullScreen() {
	if(isFullScreen())
	{
		showNormal();
		m_menuBar->setVisible(true);
	}
	else
	{
		showFullScreen();
		m_menuBar->setVisible(false);
	}
}

void MainWindow::setDirty() {
    if (m_videoControls->isTracking()) {
        imageChanged(m_player->image());
    }
    else if (!m_dirty) {
        m_dirty = true;
        QMetaObject::invokeMethod(this, "process", Qt::QueuedConnection);
    }
}

void MainWindow::process() {
    m_dirty = false;
    QImage src = m_player->image();
	m_result = src;
    imageChanged(image());
}

void MainWindow::onVideoChanged(int nframes) {
//    gpu_cache_clear();
    window()->setWindowFilePath(m_player->filename());
    window()->setWindowTitle(m_player->filename() + "[*]"); 
    actionRecord->setEnabled(nframes > 1);
}

void MainWindow::draw(ImagePaint *view, QPainter &p, const QRectF& R, const QImage& image) {
  	view->draw(p,R,image);
 }